#ifndef __Mt_Lut_Functions_H__
#define __Mt_Lut_Functions_H__

namespace Filtering { namespace MaskTools { namespace Filters { namespace Lut {

class Minimizer {

   int nMin;

public:

   Minimizer(int nValue = 255) : nMin( nValue ) {}
   void add(int nValue) { nMin = min<int>( nMin, nValue ); }
   Byte finalize() const { return nMin; }

};

class Maximizer {

   int nMax;

public:

   Maximizer(int nValue = 0) : nMax( nValue ) {}
   void add(int nValue) { nMax = max<int>( nMax, nValue ); }
   Byte finalize() const { return nMax; }

};

class Rangizer {

   int nMin, nMax;

public:

   Rangizer() : nMin( 255 ), nMax( 0 ) {}
   Rangizer(int nValue) : nMin( nValue ), nMax( nValue ) {}
   void add(int nValue) { nMin = min<int>( nMin, nValue ); nMax = max<int>( nMax, nValue ); }
   Byte finalize() const { return nMax - nMin; }

};

template<typename T>
class Averager
{

   T nSum, nCount;

public:

   Averager() : nSum( 0 ), nCount( 1 ) {}
   Averager(int nValue) : nSum( nValue ), nCount( 1 ) {}
   void add(int nValue) { nSum += nValue; nCount++; }
   Byte finalize() const { return rounded_division<T>( nSum, nCount ); }

};

template<typename T>
class Deviater
{
   T nSum, nSum2, nCount;

public:

   Deviater() : nSum( 0 ), nSum2( 0 ), nCount( 0 ) {}
   Deviater(int nValue) : nSum( nValue ), nSum2( nValue * nValue ), nCount( 1 ) {}
   void add(int nValue) { nSum += nValue; nSum2 += nValue * nValue; nCount++; }
   Byte finalize() const { return convert<Byte, Double>( sqrt( ( Double( nSum2 ) * nCount - Double( nSum ) * Double( nSum ) ) / ( Double( nCount ) * nCount ) ) ); }

};

} } } }

#endif
