#ifndef __Mt_Clamp_H__
#define __Mt_Clamp_H__

#include "../../../common/base/filter.h"

namespace Filtering { namespace MaskTools { namespace Filters { namespace Support { namespace Clamp {

typedef void(Processor)(Byte *pDst, int nDstPitch, const Byte *pSrc1, int nSrc1Pitch, const Byte *pSrc2, int nSrc2Pitch, int nWidth, int nHeight, int nOvershoot, int nUndershoot);

Processor clamp_c;
extern "C" Processor Clamp_clamp8_isse;

class Filter : public MaskTools::Filter<InPlaceFilter>
{

   int nOvershoot, nUndershoot;
   ProcessorList<Processor> processors;

protected:

   virtual void process(int n, const Plane<Byte> &dst, int nPlane)
   {
      processors.best_processor( constraints[nPlane] )( dst, dst.pitch(), frames[0].plane(nPlane), frames[0].plane(nPlane).pitch(), frames[1].plane(nPlane), frames[1].plane(nPlane).pitch(), dst.width(), dst.height(), nOvershoot, nUndershoot );
   }

public:
   Filter(const Parameters &parameters) : MaskTools::Filter<InPlaceFilter>( parameters )
   {
      nUndershoot = parameters["undershoot"];
      nOvershoot = parameters["overshoot"];

      /* add the processors */
      processors.push_back(Filtering::Processor<Processor>(&clamp_c, Constraint(CPU_NONE, 1, 1, 1, 1), 0));
      processors.push_back(Filtering::Processor<Processor>(&Clamp_clamp8_isse, Constraint(CPU_ISSE, 8, 1, 1, 1), 1));
   }

   InputConfiguration &input_configuration() const { return InPlaceThreeFrame(); }

   static Signature filter_signature()
   {
      Signature signature = "mt_clamp";

      signature.add(Parameter(TYPE_CLIP, ""));
      signature.add(Parameter(TYPE_CLIP, ""));
      signature.add(Parameter(TYPE_CLIP, ""));
      signature.add(Parameter(0, "overshoot"));
      signature.add(Parameter(0, "undershoot"));

      return add_defaults( signature );
   }

};

} } } } }

//namespace MaskTools { namespace Filters { namespace Support { namespace Clamp {
//
//typedef void(Processor)(Byte *pDst, int nDstPitch, const Byte *pSrc1, int nSrc1Pitch, const Byte *pSrc2, int nSrc2Pitch, int nWidth, int nHeight, int nOvershoot, int nUndershoot);
//
//class Filter : public MaskTools::Filter
//{
//   int nOvershoot, nUndershoot;
//protected:
//   virtual void ProcessY(PlaneProcessor *processor, Picture::Plane &dst, Picture::CPlane &src1,
//                         Picture::CPlane &src2, Picture::CPlane &src3);
//
//   virtual void ProcessU(PlaneProcessor *processor, Picture::Plane &dst, Picture::CPlane &src1,
//                         Picture::CPlane &src2, Picture::CPlane &src3);
//
//   virtual void ProcessV(PlaneProcessor *processor, Picture::Plane &dst, Picture::CPlane &src1,
//                         Picture::CPlane &src2, Picture::CPlane &src3);
//
//public:
//   Filter(const Parameters::Parameters &parameters);
//
//   static String getName();
//   static Parameters::Signature getSignature();
//   static Clip::Inputs inputClips;
//};
//
//Processor clamp_c;
//extern "C" Clamp_clamp8_mmx;
//extern "C" Clamp_clamp8_isse;
//extern "C" Clamp_clamp8_3dnow;
//
//} } } }
//
#endif